from __future__ import annotations

PACKAGE_NAME = "tach"
TOOL_NAME = "tach"
CONFIG_FILE_NAME = TOOL_NAME
PACKAGE_FILE_NAME = "package"

__all__ = ["PACKAGE_NAME", "TOOL_NAME", "CONFIG_FILE_NAME", "PACKAGE_FILE_NAME"]
